/*
 * Copyright (c) 2012-2014 Wind River Systems, Inc.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/kernel.h>
#include <nrfx_pwm.h>
#include <zephyr/irq.h>
#include <string.h>

#define STEP_COUNT_FADE 50
#define TOP_VAL_FADE    5000

static nrf_pwm_values_common_t stay_off_values[2] = { 0, 0 };
static nrf_pwm_values_common_t fade_in_out_values[2 * STEP_COUNT_FADE];

static nrf_pwm_sequence_t led_on_seq;
static nrf_pwm_sequence_t led_off_seq;



#define OUTPUT_PIN	DT_GPIO_PIN(DT_ALIAS(led0), gpios)

static nrfx_pwm_t pwm_instance = NRFX_PWM_INSTANCE(0);

static void led_softblink_start(void)
{
	nrfx_err_t err;
	uint16_t value = 0;
	uint16_t step = TOP_VAL_FADE / STEP_COUNT_FADE;

	nrfx_pwm_config_t pwm_config = NRFX_PWM_DEFAULT_CONFIG(
		OUTPUT_PIN, NRFX_PWM_PIN_NOT_USED, NRFX_PWM_PIN_NOT_USED, NRFX_PWM_PIN_NOT_USED);

	pwm_config.top_value = TOP_VAL_FADE;

	/* Connect PWM0 IRQ to nrfx_pwm_irq_handler */
	IRQ_CONNECT(DT_IRQN(DT_NODELABEL(pwm0)), DT_IRQ(DT_NODELABEL(pwm0), priority), nrfx_isr,
		    NRFX_PWM_INST_HANDLER_GET(0), 0);

	led_on_seq.values.p_common = fade_in_out_values;
	led_on_seq.end_delay = 0;
	led_on_seq.length = NRF_PWM_VALUES_LENGTH(fade_in_out_values);
	led_on_seq.repeats = 2;
	led_off_seq.values.p_common = stay_off_values;
	led_off_seq.end_delay = 0;
	led_off_seq.length = NRF_PWM_VALUES_LENGTH(stay_off_values);
	led_off_seq.repeats = 30;

	for (uint8_t i = 0; i < STEP_COUNT_FADE; ++i) {
		value += step;
		fade_in_out_values[i] = value;
		fade_in_out_values[STEP_COUNT_FADE + i] = TOP_VAL_FADE - value;
	}

	err = nrfx_pwm_init(&pwm_instance, &pwm_config, NULL, NULL);
	if (err) {
		printk("nrfx_pwm_init() failed. (err 0x%x)\n", err);
	}

	(void)nrfx_pwm_complex_playback(&pwm_instance, &led_on_seq, &led_off_seq, 1,
					NRFX_PWM_FLAG_LOOP);
}

int main(void)
{
	led_softblink_start();

	printk("Hello World! %s\n", CONFIG_BOARD);
	return 0;
}
